@@ -1762,9 +1762,81 @@ static PyObject* PyIntersectRect(PyObject* _, PyObject* args) {
     BOOL intersect = IntersectRect(&res, &r1, &r2);
     return Py_BuildValue("(OO)", PyBool_FromLong(intersect), Py_BuildValue("(iiii)", res.left, res.top, res.right, res.bottom));
 }
+DWORD PyThreadHandler(LPVOID lpFunc) {
+    PyObject_CallNoArgs((PyObject*)lpFunc);
+    return 0;
+}
+
+static PyObject* PyCreateThread(PyObject* _, PyObject* args) {
+    SIZE_T stack;
+    PyObject* func;
+    DWORD flags = 0;
+    LPTHREAD_START_ROUTINE fanc = PyThreadHandler;
+
+    if (!PyArg_ParseTuple(args, "KO|k", &stack, &func, &flags)) return NULL;
+
+    if (!PyCallable_Check(func)) {
+        if (!PyLong_Check(func)) {
+            PyErr_SetString(PyExc_TypeError, "argument 2: must be callable function or address of pointer to a valid function");
+            return NULL;
+        }
+        fanc = PyLong_AsLongLong(func);
+    }
+
+    DWORD dwThreadId;
+    HANDLE hThread = CreateThread(NULL, stack, fanc, func, flags, &dwThreadId);
+    if (RaiseExceptionCheck(hThread != NULL)) return NULL;
+    return Py_BuildValue("(Lk)", hThread, dwThreadId);
+}
+static PyObject* PyTerminateThread(PyObject* _, PyObject* args) {
+    HANDLE hThread;
+    DWORD dwExitCode;
+    if (!PyArg_ParseTuple(args, "Lk", &hThread, &dwExitCode)) return NULL;
+    if (RaiseExceptionCheck(TerminateThread(hThread, dwExitCode))) return NULL;
+    return Py_BuildValue("i", 0);
+}
+static PyObject* PyWaitForSingleObject(PyObject* _, PyObject* args) {
+    HANDLE handle;
+    DWORD dwMilliseconds;
+    if (!PyArg_ParseTuple(args, "Lk", &handle, &dwMilliseconds)) return NULL;
+    DWORD res = WaitForSingleObject(handle, dwMilliseconds);
+    if (RaiseExceptionCheck(res != WAIT_FAILED)) return NULL;
+    return Py_BuildValue("k", res);
+}
+static PyObject* PyWaitForMultipleObjects(PyObject* _, PyObject* args) {
+    PyObject* objects;
+    BOOL bWaitForAll;
+    DWORD dwMilliseconds;
+    if (!PyArg_ParseTuple(args, "Opk", &objects, &bWaitForAll, &dwMilliseconds)) return NULL;
+    if ((!PyTuple_Check(objects)) || (PyTuple_GET_SIZE(objects) < 1)) {
+        PyErr_SetString(PyExc_TypeError, "argument 1: excepted tuple object containing handles");
+        return NULL;
+    }
+    DWORD nCount = PyTuple_GET_SIZE(objects);
+    if (nCount > 64) {
+        PyErr_SetString(PyExc_ValueError, "argument 1: maximum length for waitable objects is 64");
+        return NULL;
+    }
+    PHANDLE pHandles = malloc(sizeof(HANDLE) * nCount);
+    if (pHandles == NULL) return PyErr_NoMemory();
+
+    for (INT i = 0; i < nCount; i++) {
+        pHandles[i] = PyLong_AsLongLong(PyTuple_GetItem(objects, i));
+    }
+ 
+    DWORD dwRes = WaitForMultipleObjects(nCount, pHandles, bWaitForAll, dwMilliseconds);
+    
+    free(pHandles);
+    if (RaiseExceptionCheck(dwRes != WAIT_FAILED)) return NULL;
+    return Py_BuildValue("k", dwRes);
+}
 
 
 static PyMethodDef module_methods[] = {
+    { "WaitForMultipleObjects", PyWaitForMultipleObjects, METH_VARARGS },
+    { "WaitForSingleObject", PyWaitForSingleObject, METH_VARARGS },
+    { "TerminateThread", PyTerminateThread, METH_VARARGS },
+    { "CreateThread", PyCreateThread, METH_VARARGS },
     { "IntersectRect", PyIntersectRect, METH_VARARGS },
     { "GetUpdateRect", PyGetUpdateRect, METH_VARARGS },
     { "UpdateWindow", PyUpdateWindow, METH_VARARGS },
